@@ -102,8 +102,8 @@ extern GType rb_fake_vis_get_type (void);
 
 
 typedef struct {
-	const char *name;
-	const char *displayname;
+	char *name;
+	char *displayname;
 } VisPluginInfo;
 
 typedef struct {
@@ -314,11 +314,23 @@ rb_visualizer_plugin_dispose (GObject *object)
 	G_OBJECT_CLASS (rb_visualizer_plugin_parent_class)->dispose (object);
 }
 
+static void
+free_vis_plugin_info (VisPluginInfo *vpi)
+{
+	g_free (vpi->name);
+	g_free (vpi->displayname);
+	g_free (vpi);
+}
+
 static void
 rb_visualizer_plugin_finalize (GObject *object)
 {
+	RBVisualizerPlugin *plugin = RB_VISUALIZER_PLUGIN (object);
 	rb_debug ("RBVisualizerPlugin finalising");
 
+	g_list_foreach (plugin->vis_plugin_list, (GFunc)free_vis_plugin_info, NULL);
+	g_list_free (plugin->vis_plugin_list);
+
 	G_OBJECT_CLASS (rb_visualizer_plugin_parent_class)->finalize (object);
 }
 
@@ -513,7 +525,8 @@ create_visualizer_element (const char *vis_override)
 static void
 update_playbin_visualizer (RBVisualizerPlugin *plugin,
 			   const char *vis_override,
-			   int quality)
+			   int quality,
+			   GError **error)
 {
 	GstPad *pad;
 	GstElement *vis_plugin;
@@ -582,7 +595,6 @@ tee_visualizer_inserted (RBPlayerGstTee *player, GstElement *tee, RBVisualizerPl
 							"sync-message::element",
 							G_CALLBACK (bus_sync_message_cb),
 							plugin);
-		rb_debug ("connected signal handler %d to bus %p", plugin->bus_sync_id, bus);
 	}
 
 	gst_object_unref (bus);
@@ -606,7 +618,6 @@ tee_visualizer_pre_remove (RBPlayerGstTee *player, GstElement *tee, RBVisualizer
 	bus = gst_element_get_bus (p);
 
 	if (plugin->bus_sync_id != 0) {
-		rb_debug ("disconnecting handler %d from bus %p", plugin->bus_sync_id, bus);
 		g_signal_handler_disconnect (bus, plugin->bus_sync_id);
 		plugin->bus_sync_id = 0;
 	}
@@ -617,7 +628,8 @@ tee_visualizer_pre_remove (RBPlayerGstTee *player, GstElement *tee, RBVisualizer
 static void
 update_tee_visualizer (RBVisualizerPlugin *plugin,
 		       const char *vis_override,
-		       int quality)
+		       int quality,
+		       GError **error)
 {
 	GstElement *old_vis_plugin = NULL;
 	GstPad *blocked_pad = NULL;
@@ -653,6 +665,59 @@ update_tee_visualizer (RBVisualizerPlugin *plugin,
 			rb_debug ("blocked visualizer bin sink pad");
 		}
 	} else {
+		GstStateChangeReturn state_ret;
+		GstBus *bus;
+		gboolean new_bus = FALSE;
+		gboolean failed = FALSE;
+
+		add_tee = TRUE;
+
+		/* put the sink in READY state so it grabs XV ports etc.
+		 * if it refuses to change state, disable visualization.
+		 */
+		bus = gst_element_get_bus (plugin->visualizer);
+		if (bus == NULL) {
+			bus = gst_bus_new ();
+			new_bus = TRUE;
+			gst_element_set_bus (plugin->visualizer, bus);
+		}
+
+		state_ret = gst_element_set_state (plugin->video_sink, GST_STATE_READY);
+		if (state_ret == GST_STATE_CHANGE_FAILURE) {
+			/* look for error messages on the bus */
+			while (gst_bus_have_pending (bus)) {
+				GstMessage *msg;
+
+				msg = gst_bus_pop (bus);
+				if (GST_MESSAGE_TYPE (msg) == GST_MESSAGE_ERROR) {
+					char *debug;
+
+					gst_message_parse_error (msg, error, &debug);
+					failed = TRUE;
+				}
+
+				gst_message_unref (msg);
+			}
+
+			if (failed == FALSE) {
+				g_set_error (error,
+					     RB_PLAYER_ERROR,
+					     RB_PLAYER_ERROR_GENERAL,
+					     _("Unable to start video output"));
+				failed = TRUE;
+			}
+		}
+
+		if (new_bus) {
+			gst_element_set_bus (plugin->visualizer, NULL);
+		}
+		gst_object_unref (bus);
+
+		if (failed) {
+			rb_debug ("sink failed to change state");
+			return;
+		}
+
 		add_tee = TRUE;
 	}
 
@@ -670,7 +735,12 @@ update_tee_visualizer (RBVisualizerPlugin *plugin,
 
 	if (gst_element_link_many (plugin->identity, plugin->vis_plugin, plugin->capsfilter, NULL) == FALSE) {
 		rb_debug ("failed to link in new visualizer element");
-		/* um, what now? */
+		g_set_error (error,
+			     RB_PLAYER_ERROR,
+			     RB_PLAYER_ERROR_GENERAL,
+			     _("Failed to link new visual effect into the GStreamer pipeline"));
+
+		return;
 	}
 
 	/* update the capsfilter */
@@ -703,10 +773,22 @@ update_visualizer (RBVisualizerPlugin *plugin,
 		   const char *vis_override,
 		   int quality)
 {
+	GError *error = NULL;
+
 	if (plugin->playbin_notify_id != 0) {
-		update_playbin_visualizer (plugin, vis_override, quality);
+		update_playbin_visualizer (plugin, vis_override, quality, &error);
 	} else {
-		update_tee_visualizer (plugin, vis_override, quality);
+		update_tee_visualizer (plugin, vis_override, quality, &error);
+	}
+
+	if (error != NULL) {
+		/* unfortunately recursive */
+		disable_visualization (plugin, TRUE);
+
+		rb_error_dialog (NULL,
+				 _("Unable to start visualization"),
+				 "%s",
+				 error->message);
 	}
 }
 
@@ -1401,7 +1483,7 @@ rb_visualizer_plugin_window_title_change_cb (RBShellPlayer *player,
 	char *markup;
 
 	if (title == NULL) {
-		title = "";
+		title = _("Music Player");
 	}
 
 	markup = g_markup_printf_escaped ("<big><b>%s</b></big>", title);
@@ -1543,15 +1625,11 @@ impl_activate (RBPlugin *plugin,
 					 G_CALLBACK (tee_visualizer_pre_remove),
 					 plugin, 0);
 
-		/* create the sink */
-		pi->video_sink = gst_element_factory_make ("gconfvideosink", "videosink");
-		gst_element_set_state (pi->video_sink, GST_STATE_READY);
-		find_xoverlay (pi);
 
-		/* create the rest of the bin */
 		pi->visualizer = gst_bin_new (NULL);
 		g_object_ref (pi->visualizer);
 
+		pi->video_sink = gst_element_factory_make ("gconfvideosink", "videosink");
 		pi->identity = gst_element_factory_make ("identity", NULL);
 		pi->capsfilter = gst_element_factory_make ("capsfilter", NULL);
 		colorspace = gst_element_factory_make ("ffmpegcolorspace", NULL);
@@ -1822,12 +1900,13 @@ get_vis_plugin_list (RBVisualizerPlugin *pi)
 		f = GST_ELEMENT_FACTORY (t->data);
 
 		plugin = g_new0 (VisPluginInfo, 1);
-		plugin->displayname = gst_element_factory_get_longname (f);
-		plugin->name = gst_plugin_feature_get_name (GST_PLUGIN_FEATURE (f));
+		plugin->displayname = g_strdup (gst_element_factory_get_longname (f));
+		plugin->name = g_strdup (gst_plugin_feature_get_name (GST_PLUGIN_FEATURE (f)));
 		rb_debug ("adding visualizer element: %s (%s)", plugin->displayname, plugin->name);
 
 		plugin_info = g_list_prepend (plugin_info, plugin);
 	}
+	gst_plugin_feature_list_free (features);
 	plugin_info = g_list_reverse (plugin_info);
 
 	pi->vis_plugin_list = plugin_info;
