@@ -502,20 +502,11 @@ rb_playlist_manager_error_quark (void)
 	return quark;
 }
 
-#if TOTEM_PL_PARSER_CHECK_VERSION(2,19,0)
 static void
 handle_playlist_entry_cb (TotemPlParser *playlist,
 			  const char *uri_maybe,
 			  GHashTable *metadata,
 			  RBPlaylistManager *mgr)
-#else
-static void
-handle_playlist_entry_cb (TotemPlParser *playlist,
-			  const char *uri_maybe,
-			  const char *title,
-			  const char *genre,
-			  RBPlaylistManager *mgr)
-#endif /* TOTEM_PL_PARSER_CHECK_VERSION */
 {
 	char *uri;
 #if TOTEM_PL_PARSER_CHECK_VERSION(2,19,0)
@@ -550,18 +541,28 @@ handle_playlist_entry_cb (TotemPlParser *playlist,
 }
 
 static void
-playlist_load_start_cb (TotemPlParser *parser, const char *title, RBPlaylistManager *mgr)
+playlist_load_started_cb (TotemPlParser *parser, const char *uri, GHashTable *metadata, RBPlaylistManager *mgr)
 {
-	rb_debug ("loading new playlist %s", title);
+	const char *title;
+
+	rb_debug ("loading new playlist %s", uri);
+
+	title = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_TITLE);
+	if (title == NULL)
+		title = _("Unnamed playlist");
 
 	mgr->priv->loading_playlist =
 			RB_STATIC_PLAYLIST_SOURCE (rb_playlist_manager_new_playlist (mgr, title, FALSE));
 }
 
 static void
-playlist_load_end_cb (TotemPlParser *parser, const char *title, RBPlaylistManager *mgr)
+playlist_load_ended_cb (TotemPlParser *parser, const char *uri, GHashTable *metadata, RBPlaylistManager *mgr)
 {
-	rb_debug ("finished loading playlist %s", title);
+	const char *title;
+
+	rb_debug ("finished loading playlist %s", uri);
+
+	title = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_TITLE);
 
 	if (title) {
 		g_object_set (mgr->priv->loading_playlist, "name", title, NULL);
@@ -591,22 +592,16 @@ rb_playlist_manager_parse_file (RBPlaylistManager *mgr, const char *uri, GError
 	{
 		TotemPlParser *parser = totem_pl_parser_new ();
 
-#if TOTEM_PL_PARSER_CHECK_VERSION(2,19,0)
 		g_signal_connect_object (parser, "entry-parsed",
 					 G_CALLBACK (handle_playlist_entry_cb),
 					 mgr, 0);
-#else
-		g_signal_connect_object (parser, "entry",
-					 G_CALLBACK (handle_playlist_entry_cb),
-					 mgr, 0);
-#endif /* TOTEM_PL_PARSER_CHECK_VERSION */
 
-		g_signal_connect_object (parser, "playlist-start",
-					 G_CALLBACK (playlist_load_start_cb),
+		g_signal_connect_object (parser, "playlist-started",
+					 G_CALLBACK (playlist_load_started_cb),
 					 mgr, 0);
 
-		g_signal_connect_object (parser, "playlist-end",
-					 G_CALLBACK (playlist_load_end_cb),
+		g_signal_connect_object (parser, "playlist-ended",
+					 G_CALLBACK (playlist_load_ended_cb),
 					 mgr, 0);
 
 		if (g_object_class_find_property (G_OBJECT_GET_CLASS (parser), "recurse"))
@@ -821,10 +816,10 @@ rb_playlist_manager_save_data (struct RBPlaylistManagerSaveData *data)
 }
 
 static gboolean
-model_foreach_func (GtkTreeModel *model,
-		    GtkTreePath  *path,
-		    GtkTreeIter  *iter,
-		    xmlNodePtr    root)
+save_playlist_cb (GtkTreeModel *model,
+		  GtkTreePath  *path,
+		  GtkTreeIter  *iter,
+		  xmlNodePtr    root)
 {
 	RBSource *source;
 	gboolean  local;
@@ -895,7 +890,7 @@ rb_playlist_manager_save_playlists (RBPlaylistManager *mgr, gboolean force)
 	model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (fmodel));
 	g_object_unref (fmodel);
 
-	gtk_tree_model_foreach (model, (GtkTreeModelForeachFunc)model_foreach_func, root);
+	gtk_tree_model_foreach (model, (GtkTreeModelForeachFunc)save_playlist_cb, root);
 
 	/* mark clean here.  if the save fails, we'll mark it dirty again */
 	rb_playlist_manager_set_dirty (data->mgr, FALSE);
@@ -1502,6 +1497,35 @@ rb_playlist_manager_cmd_save_playlist (GtkAction *action,
 	g_object_unref (xml);
 }
 
+static gboolean
+list_playlists_cb (GtkTreeModel *model,
+		   GtkTreePath  *path,
+		   GtkTreeIter  *iter,
+		   GList **playlists)
+{
+	RBSource *source;
+	gboolean  local;
+
+	gtk_tree_model_get (model,
+			    iter,
+			    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &source,
+			    -1);
+	if (source != NULL) {
+		if (RB_IS_PLAYLIST_SOURCE (source) && !RB_IS_PLAY_QUEUE_SOURCE (source)) {
+			
+			g_object_get (source, "is-local", &local, NULL);
+			if (local) {
+				*playlists = g_list_prepend (*playlists, source);
+			}
+		}
+		
+		g_object_unref (source);
+	}
+
+	return FALSE;
+}
+
+
 /**
  * rb_playlist_manager_get_playlists
  * @mgr: the #RBPlaylistManager
@@ -1512,7 +1536,6 @@ GList *
 rb_playlist_manager_get_playlists (RBPlaylistManager *mgr)
 {
 	GList *playlists = NULL;
-	GtkTreeIter iter;
 	GtkTreeModel *fmodel;
 	GtkTreeModel *model;
 
@@ -1520,36 +1543,8 @@ rb_playlist_manager_get_playlists (RBPlaylistManager *mgr)
 	model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (fmodel));
 	g_object_unref (fmodel);
 
-	if (gtk_tree_model_get_iter_first (model, &iter)) {
-		do {
-			RBSource *source;
-			gboolean local;
-
-			gtk_tree_model_get (model,
-					    &iter,
-					    RB_SOURCELIST_MODEL_COLUMN_SOURCE,
-					    &source,
-					    -1);
-			if (source == NULL) {
-				continue;
-			}
-			if (RB_IS_PLAYLIST_SOURCE (source) == FALSE
-			    || RB_IS_PLAY_QUEUE_SOURCE (source) == TRUE) {
-				g_object_unref (source);
-				continue;
-			}
-
-			g_object_get (source, "is-local", &local,
-				      NULL);
-			if (local) {
-				playlists = g_list_prepend (playlists, source);
-			}
-
-			g_object_unref (source);
-		} while (gtk_tree_model_iter_next (model, &iter));
-	}
-
-	return playlists;
+	gtk_tree_model_foreach (model, (GtkTreeModelForeachFunc)list_playlists_cb, &playlists);
+	return g_list_reverse (playlists);
 }
 
 /**
@@ -1568,92 +1563,78 @@ rb_playlist_manager_get_playlist_names (RBPlaylistManager *mgr,
 					gchar ***playlists,
 					GError **error)
 {
-	GtkTreeIter iter;
-	GtkTreeModel *fmodel;
-	GtkTreeModel *model;
+	GList *pl;
+	GList *t;
 	int i;
-
-	g_object_get (mgr->priv->sourcelist, "model", &fmodel, NULL);
-	model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (fmodel));
-	g_object_unref (fmodel);
-
-	if (!gtk_tree_model_get_iter_first (model, &iter)) {
-		*playlists = NULL;
-		return TRUE;
-	}
-
-	*playlists = g_new0 (char *, gtk_tree_model_iter_n_children (model, NULL) + 1);
+	
+	pl = rb_playlist_manager_get_playlists (mgr);
+	*playlists = g_new0 (char *, g_list_length (pl) + 1);
 	if (!*playlists)
 		return FALSE;
 
 	i = 0;
-	do {
-		RBSource *source;
-		char *source_name;
+	for (t = pl; t != NULL; t = t->next, i++) {
+		const char *name;
+		RBSource *source = (RBSource *)t->data;
 
-		gtk_tree_model_get (model, &iter,
-				    RB_SOURCELIST_MODEL_COLUMN_SOURCE,
-				    &source,
-				    -1);
-		if (source == NULL) {
-			continue;
-		}
-		if (RB_IS_PLAYLIST_SOURCE (source) == FALSE
-		    || RB_IS_PLAY_QUEUE_SOURCE (source) == TRUE) {
-			g_object_unref (source);
-			continue;
-		}
-
-		g_object_get (source, "name", &source_name, NULL);
-		(*playlists)[i++] = source_name;
-
-		g_object_unref (source);
-
-	} while (gtk_tree_model_iter_next (model, &iter));
+		g_object_get (source, "name", &name, NULL);
+		(*playlists)[i] = g_strdup (name);
+	}
 
 	return TRUE;
 }
 
+typedef struct {
+	const char *name;
+	RBSource *source;
+} FindPlaylistData;
+
+static gboolean
+find_playlist_by_name_cb (GtkTreeModel *model,
+			  GtkTreePath  *path,
+			  GtkTreeIter  *iter,
+			  FindPlaylistData *data)
+{
+	RBSource *source;
+
+	gtk_tree_model_get (model,
+			    iter,
+			    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &source,
+			    -1);
+	if (source != NULL) {
+		if (RB_IS_PLAYLIST_SOURCE (source) && !RB_IS_PLAY_QUEUE_SOURCE (source)) {
+			char *name;
+			
+			g_object_get (source, "name", &name, NULL);
+			if (strcmp (name, data->name) == 0) {
+				data->source = source;
+			}
+			g_free (name);
+		}
+		
+		g_object_unref (source);
+	}
+
+	return (data->source != NULL);
+}
+
 static RBSource *
 _get_playlist_by_name (RBPlaylistManager *mgr,
 		       const char *name)
 {
-	GtkTreeIter iter;
 	GtkTreeModel *fmodel;
 	GtkTreeModel *model;
-	RBSource *playlist = NULL;
+	FindPlaylistData d;
 
 	g_object_get (mgr->priv->sourcelist, "model", &fmodel, NULL);
 	model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (fmodel));
 	g_object_unref (fmodel);
 
-	if (!gtk_tree_model_get_iter_first (model, &iter))
-		return NULL;
+	d.name = name;
+	d.source = NULL;
 
-	do {
-		RBSource *source;
-		char *source_name;
-
-		gtk_tree_model_get (model, &iter,
-				    RB_SOURCELIST_MODEL_COLUMN_SOURCE,
-				    &source,
-				    -1);
-		if (source == NULL) {
-			continue;
-		}
-		if (RB_IS_PLAYLIST_SOURCE (source) == FALSE) {
-			g_object_unref (source);
-			continue;
-		}
-		g_object_get (source, "name", &source_name, NULL);
-		if (strcmp (name, source_name) == 0)
-			playlist = source;
-
-		g_free (source_name);
-		g_object_unref (source);
-	} while (gtk_tree_model_iter_next (model, &iter) && playlist == NULL);
-
-	return playlist;
+	gtk_tree_model_foreach (model, (GtkTreeModelForeachFunc)find_playlist_by_name_cb, &d);
+	return d.source;
 }
 
 /**
